@@ -1,3 +1,6 @@
+#!/usr/bin/env python
+# -*- coding: utf-8 -*-
+
 from base import *
 from record_nodes import *
 from field_nodes import *
@@ -16,6 +19,7 @@ __all__ = (
     "ValueThresholdNode",
     "BinningNode",
     "StringStripNode",
+    "DeriveNode",
     "CoalesceValueToTypeNode",
 
     # Record nodes
@@ -26,6 +30,7 @@ __all__ = (
     "AuditNode",
     "SelectNode",
     "SetSelectNode",
+    "FunctionSelectNode",
     "AuditNode",
     
     "RowListSourceNode",
