@@ -1,4 +1,8 @@
+#!/usr/bin/env python
+# -*- coding: utf-8 -*-
+
 import base
+import brewery
 import brewery.ds as ds
 import brewery.dq as dq
 import logging
@@ -122,7 +126,7 @@ class MergeNode(base.Node):
 
     To filter-out fields you do not want in your output or to rename fields you can use `maps`. It
     should be a dictionary where keys are input tags and values are either
-    :class:`brewery.ds.FieldMap` objects or dictionaries with keys ``rename`` and ``drop``.
+    :class:`brewery.FieldMap` objects or dictionaries with keys ``rename`` and ``drop``.
     
     Following example renames ``source_region_name`` field in input 0 and drops field `id` in
     input 1:
@@ -130,8 +134,8 @@ class MergeNode(base.Node):
     .. code-block:: python
 
         node.maps = {
-                        0: ds.FieldMap(rename = {"source_region_name":"region_name"}),
-                        1: ds.FieldMap(drop = ["id"])
+                        0: brewery.FieldMap(rename = {"source_region_name":"region_name"}),
+                        1: brewery.FieldMap(drop = ["id"])
                     }
 
     It is the same as:
@@ -255,8 +259,8 @@ class MergeNode(base.Node):
         if self.maps:
             for (tag, fmap) in self.maps.items():
                 if type(fmap) == dict:
-                    fmap = ds.FieldMap(rename = fmap.get("rename"), drop = fmap.get("drop"))
-                elif type(fmap) != ds.FieldMap:
+                    fmap = brewery.FieldMap(rename = fmap.get("rename"), drop = fmap.get("drop"))
+                elif type(fmap) != brewery.FieldMap:
                     raise Exception("Unknown field map type: %s" % type(fmap) )
                 f = fmap.row_filter(self.inputs[tag].fields)
                 self._maps[tag] = fmap
@@ -271,7 +275,7 @@ class MergeNode(base.Node):
             else:
                 fields += pipe.fields
 
-        self._output_fields = ds.FieldList(fields)
+        self._output_fields = brewery.FieldList(fields)
 
 
         
@@ -487,18 +491,18 @@ class AggregateNode(base.Node):
     @property
     def output_fields(self):
         # FIXME: use storage types based on aggregated field type
-        fields = ds.FieldList()
+        fields = brewery.FieldList()
 
         if self.key_fields:
             for field in  self.input_fields.fields(self.key_fields):
                 fields.append(field)
 
         for field in self.aggregated_fields:
-            fields.append(ds.Field(field + "_sum", storage_type = "float", analytical_type = "range"))
-            fields.append(ds.Field(field + "_min", storage_type = "float", analytical_type = "range"))
-            fields.append(ds.Field(field + "_max", storage_type = "float", analytical_type = "range"))
-            fields.append(ds.Field(field + "_average", storage_type = "float", analytical_type = "range"))
-        fields.append(ds.Field(self.record_count_field, storage_type = "integer", analytical_type = "range"))
+            fields.append(brewery.Field(field + "_sum", storage_type = "float", analytical_type = "range"))
+            fields.append(brewery.Field(field + "_min", storage_type = "float", analytical_type = "range"))
+            fields.append(brewery.Field(field + "_max", storage_type = "float", analytical_type = "range"))
+            fields.append(brewery.Field(field + "_average", storage_type = "float", analytical_type = "range"))
+        fields.append(brewery.Field(self.record_count_field, storage_type = "integer", analytical_type = "range"))
 
         return fields
         
@@ -561,6 +565,78 @@ class AggregateNode(base.Node):
             self.put(row)
 
 class SelectNode(base.Node):
+    """Select or discard records from the stream according to a predicate.
+
+    The parameter names of the callable function should reflect names of the fields:
+
+    .. code-block:: python
+
+        def is_big_enough(i, **args):
+            return i > 1000000
+
+        node.condition = is_big_enough
+
+    You can use ``**record`` to catch all or rest of the fields as dictionary:
+
+    .. code-block:: python
+
+        def is_big_enough(**record):
+            return record["i"] > 1000000
+            
+        node.condition = is_big_enough
+        
+
+    The condition can be also a string with python expression where local variables are record field
+    values:
+
+    .. code-block:: python
+
+        node.condition = "i > 1000000"
+
+    """
+
+    __node_info__ = {
+        "label" : "Select",
+        "description" : "Select or discard records from the stream according to a predicate.",
+        "output" : "same fields as input",
+        "attributes" : [
+            {
+                 "name": "condition",
+                 "description": "Callable or a string with python expression that will evaluate to " \
+                                "a boolean value"
+            },
+            {
+                "name": "discard",
+                 "description": "flag whether the records matching condition are discarded or included",
+                 "default": "False"
+            }
+        ]
+    }
+
+
+    def __init__(self, condition = None, discard = False):
+        """Creates and initializes selection node
+        """
+        super(SelectNode, self).__init__()
+        self.condition = condition
+        self.discard = discard
+
+    def initialize(self):
+        if isinstance(self.condition, basestring):
+            self._expression = compile(self.condition, "SelectNode condition", "eval")
+            self._condition_callable = self._eval_expression
+        else:
+            self._condition_callable = self.condition
+
+    def _eval_expression(self, **record):
+        return eval(self._expression, None, record)
+
+    def run(self):
+        for record in self.input.records():
+            if self._condition_callable(**record):
+                self.put_record(record)
+
+class FunctionSelectNode(base.Node):
     """Select records that will be selected by a predicate function.
 
 
@@ -581,8 +657,8 @@ class SelectNode(base.Node):
     """
     
     __node_info__ = {
-        "label" : "Select",
-        "description" : "Select records by a predicate function.",
+        "label" : "Function Select",
+        "description" : "Select records by a predicate function (python callable).",
         "output" : "same fields as input",
         "attributes" : [
             {
@@ -617,7 +693,7 @@ class SelectNode(base.Node):
             * `kwargs`: additional arguments passed to the function
         
         """
-        super(SelectNode, self).__init__()
+        super(FunctionSelectNode, self).__init__()
         self.function = function
         self.fields = fields
         self.discard = discard
@@ -753,7 +829,7 @@ class AuditNode(base.Node):
                                ("distinct_count", "integer", "range")
                                ]
                                
-        fields = ds.FieldList(audit_record_fields)
+        fields = brewery.FieldList(audit_record_fields)
         return fields
 
     def initialize(self):
