@@ -1,14 +1,17 @@
+#!/usr/bin/env python
+# -*- coding: utf-8 -*-
+
 import unittest
 import os
 import json
 import re
-from common import TESTS_PATH
 
 from test_data_source import *
 from test_pipes import *
 from test_nodes import *
 from test_field_list import *
 from test_node_stream import *
+from test_data_quality import *
 
 test_cases = [FieldListCase,
               DataSourceUtilsTestCase,
@@ -17,7 +20,8 @@ test_cases = [FieldListCase,
               Pipe2TestCase,
               NodesTestCase,
               StreamBuildingTestCase,
-              StreamInitializationTestCase
+              StreamInitializationTestCase,
+              DataQualityTestCase
                 ]
 
 def load_tests(loader, tests, pattern):
